import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { IAppointment } from '../form/orientation-form-view/interfaces/appointment.interface';
import { IOnlineMediation } from '../form/orientation-form-view/interfaces/onlineMediation.interface';
import { IOrientation } from '../form/orientation-form-view/interfaces/orientation.interface';
import { Structure } from '../models/structure.model';
import { User } from '../models/user.model';
import { Action, Status, Type } from '../profile/dashboard/orientation.enum';
import { StructureCategoryEnum } from '../shared/enum/structureCategory.enum';
import { StructureCategoryIconEnum } from '../shared/enum/structureCategoryIcon.enum';

type AllowedColor = 'white' | 'black' | 'red' | 'blue' | 'dark-blue' | 'grey' | 'green' | 'light-green';
@Injectable({
  providedIn: 'root',
})
export class OrientationService {
  constructor(private http: HttpClient) {}

  // User for rdv from annuaire
  public rdvUser: User;

  // Structure for rdv from structure-details
  public rdvStructure: Structure;

  /**
   * Send appointment to server
   */
  public createAppointment(newAppointment: IAppointment): Observable<IAppointment> {
    return this.http.post<IAppointment>('/api/appointment', newAppointment);
  }

  /**
   * Send online mediation to server
   */
  public createOnlineMediation(newMediation: IOnlineMediation): Observable<IOnlineMediation> {
    return this.http.post<IOnlineMediation>('/api/online-mediation', newMediation);
  }

  /**
   * Get user orientations
   */
  public getOrientationsDashboard(structureId?: string): Observable<IOrientation[]> {
    return this.http.get<IOrientation[]>('/api/orientation/dashboard', { params: structureId ? { structureId } : {} });
  }

  /**
   * Get orientation
   */
  public getOrientation(orientationId: string): Observable<IOrientation> {
    return this.http.get<IOrientation>(`/api/orientation/${orientationId}`);
  }

  /**
   * Send orientation details to server to be saved in DB
   */
  public saveOrientation(newOrientation: IOrientation): Observable<IOrientation> {
    return this.http.post<IOrientation>('/api/orientation', newOrientation);
  }

  /**
   * Patch orientation details to server to be saved in DB
   */
  public patchOrientation(orientationId: string, body: Partial<IOrientation>): Observable<IOrientation> {
    return this.http.patch<IOrientation>(`/api/orientation/${orientationId}`, body);
  }

  /**
   * Calculate days ago from date
   */
  public daysAgo(date: string | Date): number {
    const createdDate = new Date(date);
    const currentDate = new Date();
    const timeDifference = currentDate.getTime() - createdDate.getTime();
    return Math.floor(timeDifference / (1000 * 60 * 60 * 24));
  }

  /**
   * variables for dashboard
   */
  private readonly allowedColors: AllowedColor[] = [
    'white',
    'black',
    'red',
    'blue',
    'dark-blue',
    'grey',
    'green',
    'light-green',
  ];

  private readonly enumMap = {
    status: Status,
    action: Action,
    type: Type,
  };

  /**
   * Get label from orientation enum
   */
  public getTagProperty<K extends 'label' | 'color' | 'icon'>(
    category: keyof typeof this.enumMap,
    slug: string,
    property: K,
  ): K extends 'color' ? AllowedColor : string {
    const value = this.getProperty(category, slug, property);

    if (property === 'color') {
      return (this.allowedColors.includes(value as AllowedColor) ? value : 'grey') as any;
    }

    return (value || '') as any;
  }

  private getProperty(category: keyof typeof this.enumMap, slug: string, property: string): string {
    return this.enumMap[category]?.[slug as keyof (typeof this.enumMap)[typeof category]]?.[property] || '';
  }

  public getStructureTypeIcon(category): StructureCategoryIconEnum {
    switch (category) {
      case StructureCategoryEnum.public:
        return StructureCategoryIconEnum.public;
      case StructureCategoryEnum.private:
        return StructureCategoryIconEnum.private;
      case StructureCategoryEnum.privateLucrative:
        return StructureCategoryIconEnum.privateLucrative;
      default:
        return StructureCategoryIconEnum.public;
    }
  }
}
